#pragma once

#include <memory>

#include <cppmp/misc/generic_predicate.h>

#include "helper.h"
#include "../types/type.h"

namespace cpprtti
{

    namespace __impl
    {

        /* make_variant_impl */

        template<typename X, typename = void>
        struct variant_builder;

        /* variant_impl */

        struct variant_impl
        {
            const type&     type;
            void *          data            { nullptr };
            bool            is_const        { false };
            bool            is_reference    { false };

            /**
             * @brief Constructor.
             */
            inline variant_impl(const cpprtti::type& p_type);
        };

        using variant_impl_ptr_s = std::shared_ptr<const variant_impl>;

    }

    struct variant
    {
    public:
        template<typename X, typename>
        friend struct __impl::variant_builder;

    private:
        __impl::variant_impl_ptr_s  _impl;  //!< Implementation of the variant

        /**
         * @brief Constructor.
         */
        inline variant(__impl::variant_impl_ptr_s p_impl);

    public:
        /**
         * @brief Default constructor.
         */
        inline variant();

        /**
         * @brief Get type information of the stored value.
         */
        inline const type& type() const;

        /**
         * @brief Check if the stored type is a reference.
         */
        inline bool is_reference() const;

        /**
         * @brief Check if the stored type is a constant.
         */
        inline bool is_const() const;

        /**
         * @brief Get the stored value.
         */
        template<typename T>
        inline T get() const;

        /**
         * @brief Check if the variant contains a value.
         */
        inline bool empty();

        /**
         * @brief Check if the variant contains a value.
         */
        inline operator bool() const;

    private:
        /**
         * @brief Check if the variant has a value assigned, if not throw an exception.
         */
        inline void check_impl() const;
    };

    constexpr decltype(auto) make_variant = cppmp::generic_predicate<__impl::variant_builder> { };

}

#include "variant.inl"
