#pragma once

#include <map>
#include <memory>

#include "type.h"
#include "member/member.h"

namespace cpprtti
{

    struct class_type
        : public type
    {
    private:
        using member_ptr_u = std::unique_ptr<member>;
        using member_map   = std::map<std::string, member_ptr_u>;

    private:
        member_map _members;

    public:
        /**
         * @brief Constructor.
         */
        inline class_type(
            cpprtti::registry&  p_registry,
            type_id             p_id,
            const std::string&  p_name);

        /**
         * @brief Get a map of all registered members.
         */
        const member_map& members() const;

        /**
         * @brief Register a new member variable.
         */
        template<typename... T_args>
        auto& add_member_variable(
            const std::string&  p_name,
            T_args&&...         p_args);

        /**
         * @brief Register a new member method.
         */
        template<typename... T_args>
        auto& add_member_method(
            const std::string&  p_name,
            T_args&&...         p_args);
    };

}

#include "class_type.inl"
