#pragma once

#include <string>

#include <cpprtti/misc/helper.h>

namespace cpprtti
{

    struct registry;

    enum class rtti_type
    {
        unknown = 0,        //!< unknown
        fundamental_type,   //!< fundamental C++ type (like int, float, ...)
        class_type,         //!< C++ class/struct
    };

    using rtti_type_t = enum rtti_type;

    /**
     * @brief represents any C++ type
     */
    struct type
    {
    protected:
        cpprtti::registry&  _registry;      //!< Type registry this type belongs to.
        type_id             _id;            //!< Unique ID
        std::string         _name;          //!< Name of the type.
        rtti_type_t         _rtti_type;     //!< Type specialization

    public:
        /**
         * @brief Constructor.
         */
        inline type(
            cpprtti::registry&  p_registry,
            size_t              p_id,
            const std::string&  p_name,
            rtti_type_t         p_rtti_type);

        /**
         * @brief Destructor.
         */
        virtual ~type() = default;

        /**
         * @brief Type registry this type belongs to.
         */
        inline cpprtti::registry& registry();

        /**
         * @brief Type registry this type belongs to.
         */
        inline const cpprtti::registry& registry() const;

        /**
         * @brief Get the ID of this type.
         */
        inline type_id id() const;

        /**
         * @brief Get the name of this type.
         */
        inline const std::string& name() const;

        /**
         * @brief Get the RTTI type of this type object.
         */
        inline rtti_type_t rtti_type() const;
    };

}

#include "type.inl"
